@@ -242,10 +242,8 @@ struct request {
 	size_t				 buf_len;
 
 	uint8_t				 outbuf[GOTWEBD_CACHESIZE];
-	size_t				 outbuf_len;
 
 	char				 querystring[MAX_QUERYSTRING];
-	char				 http_host[GOTWEBD_MAXTEXT];
 	char				 document_uri[MAX_DOCUMENT_URI];
 	char				 server_name[MAX_SERVER_NAME];
 	int				 https;
@@ -498,21 +496,15 @@ void fcgi_timeout(int, short, void *);
 void fcgi_cleanup_request(struct request *);
 void fcgi_create_end_record(struct request *);
 void dump_fcgi_record(const char *, struct fcgi_record_header *);
-int fcgi_puts(struct template *, const char *);
-int fcgi_putc(struct template *, int);
-int fcgi_vprintf(struct request *, const char *, va_list);
-int fcgi_printf(struct request *, const char *, ...)
-	__attribute__((__format__(printf, 2, 3)))
-	__attribute__((__nonnull__(2)));
-int fcgi_gen_binary_response(struct request *, const uint8_t *, int);
+int fcgi_write(void *, const void *, size_t);
 
 /* got_operations.c */
 const struct got_error *got_gotweb_closefile(FILE *);
 const struct got_error *got_get_repo_owner(char **, struct request *);
 const struct got_error *got_get_repo_age(time_t *, struct request *,
     const char *);
-const struct got_error *got_get_repo_commits(struct request *, int);
-const struct got_error *got_get_repo_tags(struct request *, int);
+const struct got_error *got_get_repo_commits(struct request *, size_t);
+const struct got_error *got_get_repo_tags(struct request *, size_t);
 const struct got_error *got_get_repo_heads(struct request *);
 const struct got_error *got_open_diff_for_output(FILE **, struct request *);
 int got_output_repo_tree(struct request *,
