@@ -14,6 +14,8 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#include "got_compat.h"
+
 #include <sys/types.h>
 #include <sys/queue.h>
 #include <sys/uio.h>
@@ -109,6 +111,12 @@ got_repo_get_fd(struct got_repository *repo)
 	return repo->gitdir_fd;
 }
 
+enum got_hash_algorithm
+got_repo_get_object_format(struct got_repository *repo)
+{
+	return repo->algo;
+}
+
 const char *
 got_repo_get_gitconfig_author_name(struct got_repository *repo)
 {
@@ -860,6 +868,8 @@ got_repo_close(struct got_repository *repo)
 		imsg_clear(repo->privsep_children[i].ibuf);
 		free(repo->privsep_children[i].ibuf);
 		err = got_privsep_send_stop(repo->privsep_children[i].imsg_fd);
+		if (err && err->code == GOT_ERR_EOF)
+			err = NULL;
 		child_err = got_privsep_wait_for_child(
 		    repo->privsep_children[i].pid);
 		if (child_err && err == NULL)
@@ -893,11 +903,115 @@ got_repo_close(struct got_repository *repo)
 	return err;
 }
 
+const struct got_error *
+got_repo_remote_repo_dup(struct got_remote_repo **newp,
+    const struct got_remote_repo *repo)
+{
+	const struct got_error *err = NULL;
+	struct got_remote_repo *new;
+	int i;
+
+	new = calloc(1, sizeof(*new));
+	if (new == NULL)
+		return got_error_from_errno("calloc");
+	
+	if (repo->name) {
+		new->name = strdup(repo->name);
+		if (new->name == NULL) {
+			err = got_error_from_errno("strdup");
+			goto done;
+		}
+	}
+
+	if (repo->fetch_url) {
+		new->fetch_url = strdup(repo->fetch_url);
+		if (new->fetch_url == NULL) {
+			err = got_error_from_errno("strdup");
+			goto done;
+		}
+	}
+
+	if (repo->send_url) {
+		new->send_url = strdup(repo->send_url);
+		if (new->send_url == NULL) {
+			err = got_error_from_errno("strdup");
+			goto done;
+		}
+	}
+
+	new->mirror_references = repo->mirror_references;
+
+	new->nfetch_branches = repo->nfetch_branches;
+	if (repo->fetch_branches) {
+		new->fetch_branches = calloc(repo->nfetch_branches,
+		    sizeof(char *));
+		if (new->fetch_branches == NULL) {
+			err = got_error_from_errno("calloc");
+			goto done;
+		}
+		for (i = 0; i < repo->nfetch_branches; i++) {
+			new->fetch_branches[i] = strdup(
+			    repo->fetch_branches[i]);
+			if (new->fetch_branches[i] == NULL) {
+				err = got_error_from_errno("strdup");
+				goto done;
+			}
+		}
+	}
+
+	new->nsend_branches = repo->nsend_branches;
+	if (repo->send_branches) {
+		new->send_branches = calloc(repo->nsend_branches,
+		    sizeof(char *));
+		if (new->send_branches == NULL) {
+			err = got_error_from_errno("calloc");
+			goto done;
+		}
+		for (i = 0; i < repo->nsend_branches; i++) {
+			new->send_branches[i] = strdup(
+			    repo->send_branches[i]);
+			if (new->send_branches[i] == NULL) {
+				err = got_error_from_errno("strdup");
+				goto done;
+			}
+		}
+	}
+
+	new->nfetch_refs = repo->nfetch_refs;
+	if (repo->fetch_refs) {
+		new->fetch_refs = calloc(repo->nfetch_refs,
+		    sizeof(char *));
+		if (new->fetch_refs == NULL) {
+			err = got_error_from_errno("calloc");
+			goto done;
+		}
+		for (i = 0; i < repo->nfetch_refs; i++) {
+			new->fetch_refs[i] = strdup(
+			    repo->fetch_refs[i]);
+			if (new->fetch_refs[i] == NULL) {
+				err = got_error_from_errno("strdup");
+				goto done;
+			}
+		}
+	}
+done:
+	if (err) {
+		got_repo_free_remote_repo_data(new);
+		free(new);
+	} else
+		*newp = new;
+
+	return err;
+}
+
 void
 got_repo_free_remote_repo_data(struct got_remote_repo *repo)
 {
 	int i;
 
+	if (repo == NULL)
+		return;
+
 	free(repo->name);
 	repo->name = NULL;
 	free(repo->fetch_url);
@@ -914,6 +1028,11 @@ got_repo_free_remote_repo_data(struct got_remote_repo *repo)
 	free(repo->send_branches);
 	repo->send_branches = NULL;
 	repo->nsend_branches = 0;
+	for (i = 0; i < repo->nfetch_refs; i++)
+		free(repo->fetch_refs[i]);
+	free(repo->fetch_refs);
+	repo->fetch_refs = NULL;
+	repo->nfetch_refs = 0;
 }
 
 const struct got_error *
@@ -1733,7 +1852,6 @@ match_loose_object(struct got_object_id **unique_id, const char *path_objects,
 	}
 	while ((dent = readdir(dir)) != NULL) {
 		int cmp;
-		enum got_hash_algorithm algo = GOT_HASH_SHA1;
 
 		free(id_str);
 		id_str = NULL;
@@ -1747,7 +1865,7 @@ match_loose_object(struct got_object_id **unique_id, const char *path_objects,
 			goto done;
 		}
 
-		if (!got_parse_object_id(&id, id_str, algo))
+		if (!got_parse_object_id(&id, id_str, repo->algo))
 			continue;
 
 		/*
@@ -2287,7 +2405,6 @@ got_repo_get_loose_object_info(int *nobjects, off_t *ondisk_size,
 			char *id_str;
 			int fd;
 			struct stat sb;
-			enum got_hash_algorithm algo = GOT_HASH_SHA1;
 
 			if (strcmp(dent->d_name, ".") == 0 ||
 			    strcmp(dent->d_name, "..") == 0)
@@ -2298,7 +2415,7 @@ got_repo_get_loose_object_info(int *nobjects, off_t *ondisk_size,
 				goto done;
 			}
 
-			if (!got_parse_object_id(&id, id_str, algo)) {
+			if (!got_parse_object_id(&id, id_str, repo->algo)) {
 				free(id_str);
 				continue;
 			}
